<?php
declare(strict_types=1);

if (!defined('_PS_VERSION_')) {
    exit;
}

include_once(_PS_MODULE_DIR_ . 'conotoxiapay/tools/sdk/internals/autoload.php');
include_once(_PS_MODULE_DIR_ . 'conotoxiapay/tools/class-conotoxia-pay-storage.php');

use CKPL\Pay\Configuration\ConfigurationInterface;
use CKPL\Pay\Configuration\Factory\ConfigurationFactory;
use CKPL\Pay\Exception\ClientException;
use CKPL\Pay\Exception\ConfigurationException;
use CKPL\Pay\Model\Response\CreatedRefundResponseModel;
use CKPL\Pay\Pay;
use PrestaShop\Module\ConotoxiaPay\Configure\Configurator;
use PrestaShop\Module\ConotoxiaPay\Definitions;
use PrestaShop\Module\ConotoxiaPay\Install\InstallerFactory;
use PrestaShop\Module\ConotoxiaPay\Repository\PaymentRepository;
use PrestaShop\Module\ConotoxiaPay\Repository\RefundRepository;
use PrestaShop\PrestaShop\Core\Localization\Exception\LocalizationException;
use PrestaShop\PrestaShop\Core\Payment\PaymentOption;

require_once __DIR__ . '/vendor/autoload.php';

class ConotoxiaPay extends PaymentModule
{

    /**
     * @type string
     */
    const PAYMENTS_HOST = 'https://partner.cinkciarz.pl';

    /**
     * @type string
     */
    const OIDC_HOST = 'https://login.cinkciarz.pl';

    /**
     * @type string
     */
    const SANDBOX_PAYMENTS_HOST = 'https://pay-api.ckpl.us';

    /**
     * @type string
     */
    const SANDBOX_OIDC_HOST = 'https://oidc.ckpl.us';

    /**
     * @var string[][]
     */
    private $paymentMethodIcons = array(
//        array('name' => 'PrzelewOnline', 'title' => 'Przelew Online'),
        array('name' => 'Blik', 'title' => 'BLIK'),
        array('name' => 'Visa', 'title' => 'Visa'),
        array('name' => 'MasterCard', 'title' => 'MasterCard'),
        array('name' => 'GooglePay', 'title' => 'Google Pay'),
        array('name' => 'ApplePay', 'title' => 'Apple Pay'),
        array('name' => 'PayPal', 'title' => 'PayPal'),
    );

    /**
     * ConotoxiaPay constructor.
     * @throws Exception
     */
    public function __construct()
    {
        $this->name = Definitions::MODULE_NAME;
        $this->tab = 'payments_gateways';
        $this->version = '1.35.0';
        $this->ps_versions_compliancy = ['min' => '1.7.7', 'max' => _PS_VERSION_];
        $this->author = $this->l('Company name');
        $this->need_instance = 1;
        $this->bootstrap = true;
        $this->limited_countries = [];

        parent::__construct();

        $this->displayName = $this->l('Conotoxia Pay');
        $this->description = $this->l('Plugin description');
        $this->confirmUninstall = $this->l('Are you sure you want to uninstall?');

        foreach ($this->getConfigurationData() as $key => $value) {
            if (empty($value)) {
                $this->warning = $this->l('Missing configuration for ') . ' "' . $key . '" ' . $this->l('field');
                return;
            }
        }
        if (!count(Currency::checkPaymentCurrencies($this->id))) {
            $this->warning = $this->l('No currency has been set for this module.');
        }
    }


    /**
     * @param $hookParams
     * @return array|void
     * @throws Exception
     */
    public function hookPaymentOptions($params)
    {
        if (!$this->active) {
            return;
        }

        $paymentOption = (new PaymentOption())
            ->setCallToActionText($this->l('Pay conveniently online with Conotoxia Pay'))
            ->setAdditionalInformation($this->getPaymentAdditionalInfo())
            ->setModuleName($this->name)
            ->setAction($this->context->link->getModuleLink($this->name, 'payment'));
        if ($this->getConfigurationValue(Definitions::ICON)) {
            $paymentOption->setLogo($this->getLogo());
        }

        return [$paymentOption];
    }

    private function getPaymentAdditionalInfo() {
        $paymentMethodIconsToDisplay = array();
        $paymentMethodIcons = $this->getPaymentMethodIconsConfiguration();

        foreach ($this->paymentMethodIcons as $paymentMethodIcon) {
            if (in_array($paymentMethodIcon['name'], $paymentMethodIcons) ) {
                $paymentMethodIconsToDisplay[] = $paymentMethodIcon;
            }
        }

        $this->context->smarty->assign([
            'paymentMethodIcons' => $paymentMethodIconsToDisplay,
            'conotoxiapayModuleDir' => $this->_path
        ]);
        return $this->context->smarty->fetch('module:conotoxiapay/views/templates/front/payment_methods.tpl');
    }

    public function getLogo($file = 'ConotoxiaPay.svg')
    {
        return Media::getMediaPath(_PS_MODULE_DIR_ . $this->name . '/views/img/' . $file);
    }

    /**
     * Module configuration form
     *
     * @return string
     * @throws PrestaShopException
     * @throws Exception
     */
    public function getContent()
    {
        $html = '';
        $publicKey = null;
        $privateKey = null;

        if (isset($_SESSION['generate_keys']) && Tools::getValue('generate_keys') === $_SESSION['generate_keys']) {
            $keys = $this->handleGenerateKeys($html);
            $publicKey = $keys['public'];
            $privateKey = $keys['private'];
        }
        if (Tools::isSubmit('btnSubmit')) {
            $privateKey = trim(Tools::getValue(Definitions::PRIVATE_KEY));
            $publicKey = $this->getPublicKey($privateKey);

            $this->setConfigurationValue(Definitions::CLIENT_ID, trim(Tools::getValue(Definitions::CLIENT_ID)));
            $clientSecret = Tools::getValue(Definitions::CLIENT_SECRET);
            if ($clientSecret) {
                $this->setConfigurationValue(Definitions::CLIENT_SECRET, trim($clientSecret));
            }
            $this->setConfigurationValue(Definitions::POINT_OF_SALE, trim(Tools::getValue(Definitions::POINT_OF_SALE)));
            $this->setConfigurationValue(Definitions::PUBLIC_KEY, $publicKey);
            $this->setConfigurationValue(Definitions::PRIVATE_KEY, $privateKey);
            $this->setConfigurationValue(Definitions::SANDBOX, Tools::getValue(Definitions::SANDBOX));
            $this->setConfigurationValue(Definitions::ICON, Tools::getValue(Definitions::ICON));
            $this->setConfigurationValue(Definitions::PAYMENT_METHOD_ICONS, $this->getChosenPaymentMethodIcons());

            if ($this->isMinimalConfigurationCompleted()) {
                try {
                    $sdk = $this->initializeConotoxiaPay();
                    $sdk->merchant()->invalidateToken();
                    $sdk->pickSignatureKey();
                    $addedKeyResponse = $sdk->merchant()->sendPublicKey();
                    if ($addedKeyResponse->getStatus() !== 'ACTIVATED') {
                        $generateKeysMessage = $this->l('The generated public key is inactive. To complete the configuration, activate it in the');
                        $merchantPanel = $this->l('Conotoxia Merchant Panel.');
                        $merchantPanelUrl = $this->l('https://fx.conotoxia.com/merchant/configuration');
                        $html .= $this->displayWarning($generateKeysMessage . ' ' . "<a href='$merchantPanelUrl'>$merchantPanel</a>");
                    }

                    if ($this->isConfigurationCompleted()) {
                        $html .= $this->displayConfirmation($this->l('Saved public key in Conotoxia Pay.'));
                    } else {
                        $html .= $this->displayInformation($this->l('Saved public key in Conotoxia Pay but some of the fields are missing.'));
                    }
                } catch (CKPL\Pay\Exception\Api\PublicKeyAlreadyExistException $e) {
                    $sdk->merchant()->setPublicKeyId($e->getKid());
                    if ($this->isConfigurationCompleted()) {
                        $html .= $this->displayConfirmation($this->l('Settings saved.'));
                    } else {
                        $html .= $this->displayInformation($this->l('Settings saved but some of the fields are missing.'));
                    }
                } catch (Exception $e) {
                    PrestaShopLogger::addLog('ConotoxiaPay - saving configuration problem: ' . $e->getMessage(), 2, $e->getCode(), 'Pay', null, true);
                    $html .= $this->displayWarning($this->getSettingsProblemMessage());
                }
            } else {
                if (!empty($privateKey) && $publicKey === null) {
                    $html .= $this->displayWarning($this->l('Wrong private key.'));
                } else {
                    $html .= $this->displayInformation($this->l('Settings saved but some of the fields are missing.'));
                }
            }
        }

        $generationToken = (string)mt_rand();
        $_SESSION['generate_keys'] = $generationToken;

        $fields_form = array(
            'form' => array(
                'legend' => array(
                    'title' => $this->l('Conotoxia Pay configurator'),
                    'icon' => 'icon-envelope'
                ),
                'input' => array(
                    array(
                        'type' => 'text',
                        'label' => $this->l('Client id'),
                        'name' => Definitions::CLIENT_ID,
                        'required' => true,
                        'hint' => $this->l('API Client identifier can be generated in the Merchant\'s Panel on Configuration page.'),
                    ),
                    array(
                        'type' => 'password',
                        'label' => $this->l('Client secret'),
                        'name' => Definitions::CLIENT_SECRET,
                        'required' => true,
                        'hint' => $this->l('API Client secret can be generated in the Merchant\'s Panel on Configuration page.'),
                    ),
                    array(
                        'type' => 'text',
                        'label' => $this->l('Point of sale'),
                        'name' => Definitions::POINT_OF_SALE,
                        'required' => true,
                        'hint' => $this->l('Point of Sale ID is generated by the Conotoxia Pay system in the Merchant\'s Panel (e.g. POS1234567898765432).'),
                    ),
                    array(
                        'type' => 'textarea',
                        'label' => $this->l('RSA private key'),
                        'name' => Definitions::PRIVATE_KEY,
                        'required' => true,
                        'hint' => $this->l('RSA private key that will be used to sign request.'),
                        'desc' => nl2br($this->generateModuleConfigurationLink(['generate_keys' => $generationToken], $this->l('Generate public and private key'))),
                    ),
                    array(
                        'type' => 'text',
                        'label' => $this->l('RSA public key identifier'),
                        'name' => Definitions::PUBLIC_KEY_ID,
                        'readonly' => true,
                        'hint' => nl2br($this->l('RSA public key identifier.')),
                    ),
                    array(
                        'type' => 'textarea',
                        'label' => $this->l('RSA public key'),
                        'name' => Definitions::PUBLIC_KEY,
                        'readonly' => true,
                        'hint' => nl2br($this->l('RSA public key that will be sent to Conotoxia Pay.')),
                    ),
                    array(
                        'type' => 'switch',
                        'label' => $this->l('Sandbox mode'),
                        'name' => Definitions::SANDBOX,
                        'required' => true,
                        'values' => [['id' => 'active_on',
                            'value' => 1,
                            'label' => $this->l('Enabled')],
                            ['id' => 'active_off',
                                'value' => 0,
                                'label' => $this->l('Disabled')]],
                    ),
                    array(
                        'type' => 'switch',
                        'label' => $this->l('Payment icon'),
                        'name' => Definitions::ICON,
                        'required' => false,
                        'values' => [['id' => 'active_on',
                            'value' => 1,
                            'label' => $this->l('Enabled')],
                            ['id' => 'active_off',
                                'value' => 0,
                                'label' => $this->l('Disabled')]],
                    ),
                    array(
                        'type'    => 'checkbox',
                        'label'   => $this->l('Payment method icons'),
                        'name'    => Definitions::PAYMENT_METHOD_ICONS,
                        'values'  => array(
                            'query' => $this->paymentMethodIcons,
                            'id'    => 'name',
                            'name'  => 'title',
                        ),
                    ),
                ),
                'submit' => ['title' => $this->l('Save'),
                ]
            ),
        );

        $helper = new HelperForm();
        $helper->name_controller = $this->name;
        $helper->token = Tools::getAdminTokenLite('AdminModules');
        $helper->show_toolbar = false;
        $helper->title = $this->displayName;
        $helper->submit_action = 'btnSubmit';
        $helper->tpl_vars = ['fields_value' => $this->getConfigFieldsValues($privateKey, $publicKey)];

        return $html . $helper->generateForm([$fields_form]);
    }

    private function getChosenPaymentMethodIcons() {
        $chosenPaymentMethodIcons = array();
        foreach ($this->paymentMethodIcons as $paymentMethodIcon) {
            if (Tools::getValue(Definitions::PAYMENT_METHOD_ICONS.'_'.$paymentMethodIcon['name'])) {
                $chosenPaymentMethodIcons[] = $paymentMethodIcon['name'];
            }
        }
        return implode(',', $chosenPaymentMethodIcons);
    }

    /**
     * @param string $html
     * @return array
     */
    private function handleGenerateKeys(&$html)
    {
        if (!extension_loaded('openssl')) {
            $html .= $this->displayWarning($this->l('Missing OpenSSL extension.'));
            PrestaShopLogger::addLog('ConotoxiaPay - missing OpenSSL extension', 3, null, 'Pay', null, true);
            return ['public' => null, 'private' => null];
        }

        $keyPair = openssl_pkey_new(['private_key_bits' => 2048, 'private_key_type' => OPENSSL_KEYTYPE_RSA]);
        openssl_pkey_export($keyPair, $privateKey);
        $publicKey = openssl_pkey_get_details($keyPair) ['key'];
        openssl_pkey_free($keyPair);
        $html .= $this->displayInformation($this->l('Remember to save configuration after generating keys.'));
        return ['public' => trim($publicKey), 'private' => trim($privateKey)];
    }

    /**
     * @param string $privateKey
     * @return string|null
     */
    private function getPublicKey($privateKey)
    {
        if (!extension_loaded('openssl')) {
            PrestaShopLogger::addLog('ConotoxiaPay - missing OpenSSL extension', 3, null, 'Pay', null, true);
            return null;
        }
        $keyPair = openssl_pkey_get_private($privateKey);
        if ($keyPair === false) {
            return null;
        }
        $publicKey = openssl_pkey_get_details($keyPair) ['key'];
        openssl_pkey_free($keyPair);
        return trim($publicKey);
    }

    /**
     * @param array $required
     * @return bool
     * @throws Exception
     */
    public function isConfigurationCompleted($required = [Definitions::CLIENT_ID, Definitions::CLIENT_SECRET, Definitions::POINT_OF_SALE, Definitions::PUBLIC_KEY, Definitions::PRIVATE_KEY])
    {
        $result = true;

        foreach ($required as $option) {
            if (empty($this->getConfigurationValue($option))) {
                $result = false;
                break;
            }
        }

        return $result;
    }

    /**
     * @return bool
     * @throws Exception
     */
    public function isMinimalConfigurationCompleted()
    {
        return $this->isConfigurationCompleted([Definitions::CLIENT_ID, Definitions::CLIENT_SECRET, Definitions::PUBLIC_KEY, Definitions::PRIVATE_KEY]);
    }

    /**
     * @return CKPL\Pay\Pay
     * @throws Exception
     */
    public function initializeConotoxiaPay()
    {
        try {
            return new Pay($this->makeConfiguration());
        } catch (Exception $e) {
            PrestaShopLogger::addLog('ConotoxiaPay - initialization problem: ' . $e->getMessage(), 3, $e->getCode(), 'Pay', null, true);
            throw $e;
        }
    }

    /**
     * @return ConfigurationInterface
     * @throws ConfigurationException
     * @throws Exception
     */
    protected function makeConfiguration()
    {
        return ConfigurationFactory::fromArray([
            ConfigurationInterface::HOST => $this->getPaymentHost(),
            ConfigurationInterface::OIDC => $this->getOidcHost(),
            ConfigurationInterface::CLIENT_ID => $this->getConfigurationValue(Definitions::CLIENT_ID),
            ConfigurationInterface::CLIENT_SECRET => $this->getConfigurationValue(Definitions::CLIENT_SECRET),
            ConfigurationInterface::POINT_OF_SALE => $this->getConfigurationValue(Definitions::POINT_OF_SALE),
            ConfigurationInterface::PRIVATE_KEY => $this->getConfigurationValue(Definitions::PRIVATE_KEY),
            ConfigurationInterface::PUBLIC_KEY => $this->getConfigurationValue(Definitions::PUBLIC_KEY),
            ConfigurationInterface::STORAGE => new PS_Gateway_Conotoxia_Pay_Storage($this->get('prestashop.adapter.legacy.configuration'))
        ]);
    }

    /**
     * @return string
     * @throws Exception
     */
    private function getPaymentHost()
    {
        if ($this->getConfigurationValue(Definitions::SANDBOX)) {
            return static::SANDBOX_PAYMENTS_HOST;
        }
        return static::PAYMENTS_HOST;
    }

    /**
     * @return string
     * @throws Exception
     */
    private function getOidcHost()
    {
        if ($this->getConfigurationValue(Definitions::SANDBOX)) {
            return static::SANDBOX_OIDC_HOST;
        }
        return static::OIDC_HOST;
    }

    /**
     * @return string
     * @throws PrestaShopException
     */
    private function getSettingsProblemMessage()
    {
        return sprintf($this->l('Settings saved with errors.'), $this->generateLogsLink($this->l('Logs')));
    }

    /**
     * @param $linkText string
     * @return string
     * @throws PrestaShopException
     */
    private function generateLogsLink($linkText)
    {
        return $this->generateLink($this
            ->context
            ->link
            ->getAdminLink('AdminLogs', true), $linkText);
    }

    /**
     * @param string $address
     * @param string $label
     * @return string
     */
    private function generateLink($address, $label)
    {
        return '<a href="' . $address . '">' . $label . '</a>';
    }

    /**
     * @param array $queryParameters
     * @param string $linkText
     * @return string
     * @throws PrestaShopException
     */
    private function generateModuleConfigurationLink($queryParameters, $linkText)
    {
        return $this->generateLink($this
            ->context
            ->link
            ->getAdminLink('AdminModules', true, [], array_merge(['configure' => Definitions::MODULE_NAME], $queryParameters)), $linkText);
    }

    /**
     * @param null $generatedPrivateKey
     * @param null $generatedPublicKey
     * @return array
     * @throws Exception
     */
    public function getConfigFieldsValues($generatedPrivateKey = null, $generatedPublicKey = null)
    {
        $fieldsValues = [
            Definitions::CLIENT_ID => Tools::getValue(Definitions::CLIENT_ID, $this->getConfigurationValue(Definitions::CLIENT_ID)),
            Definitions::CLIENT_SECRET => Tools::getValue(Definitions::CLIENT_SECRET, $this->getConfigurationValue(Definitions::CLIENT_SECRET)),
            Definitions::POINT_OF_SALE => Tools::getValue(Definitions::POINT_OF_SALE, $this->getConfigurationValue(Definitions::POINT_OF_SALE)),
            Definitions::PUBLIC_KEY_ID => $this->getConfigurationValue(Definitions::PUBLIC_KEY_ID),
            Definitions::PUBLIC_KEY => $generatedPublicKey ?? $this->getConfigurationValue(Definitions::PUBLIC_KEY),
            Definitions::PRIVATE_KEY => $generatedPrivateKey ?? $this->getConfigurationValue(Definitions::PRIVATE_KEY),
            Definitions::SANDBOX => Tools::getValue(Definitions::SANDBOX, $this->getConfigurationValue(Definitions::SANDBOX)),
            Definitions::ICON => Tools::getValue(Definitions::ICON, $this->getConfigurationValue(Definitions::ICON)),
        ];
        foreach ($this->getPaymentMethodIconsConfiguration() as $paymentMethodIcon) {
            $fieldsValues[Definitions::PAYMENT_METHOD_ICONS.'_'.$paymentMethodIcon] = true;
        }
        return $fieldsValues;
    }

    private function getPaymentMethodIconsConfiguration() {
        if ($paymentMethodIcons = $this->getConfigurationValue(Definitions::PAYMENT_METHOD_ICONS)) {
            return $paymentMethodIcons = explode(',', $this->getConfigurationValue(Definitions::PAYMENT_METHOD_ICONS));
        } else {
            return array();
        }
    }

    /**
     * @return bool
     * @throws PrestaShopException
     */
    public function install()
    {
        if (!parent::install()) {
            return false;
        }
        return InstallerFactory::create()->install($this);
    }

    /**
     * @return bool
     * @throws PrestaShopException
     */
    public function uninstall()
    {
        return InstallerFactory::create()->uninstall($this) && parent::uninstall();
    }

    /**
     * Used for adding new order state translation when new language is installed
     *
     * @param $params
     */
    public function hookActionUpdateLangAfter($params)
    {
        if (!empty($params['lang']) && $params['lang'] instanceof Language) {
            try {
                Configurator::updateOrderWaitingStatusLanguage($params['lang']);
            } catch (Exception $e) {
                PrestaShopLogger::addLog('ConotoxiaPay - cannot update translation for module waiting status: ' . $e->getMessage(), 3, $e->getCode(), null, null, true);
            }
        }
    }

    /**
     * @param $params
     * @return array[]
     */
    public function hookModuleRoutes($params): array
    {
        return array(
            'notification' => array(
                'controller' => 'notification',
                'rule' => 'conotoxiapay-notification',
                'keywords' =>  array(),
                'params' => array(
                    'fc' => 'module',
                    'module' => 'conotoxiapay',
                    'controller' => 'notification'
                )
            ),
            'success' => array(
                'controller' => 'success',
                'rule' => 'conotoxiapay-success',
                'keywords' =>  array(),
                'params' => array(
                    'fc' => 'module',
                    'module' => 'conotoxiapay',
                    'controller' => 'success'
                )
            )
        );
    }

    /**
     * @throws PrestaShopDatabaseException
     * @throws PrestaShopException
     */
    public function hookDisplayAdminOrderTabLink($params)
    {
        if (!$this->showRefundsPanel($params)) {
            return null;
        }

        $refunds = RefundRepository::getRefundsByOrderId($params['id_order']);

        $orderId = $params['id_order'];
        $order = new Order($orderId);
        $paymentId = $this->getPaymentIdFromOrder($order);

        $this->context->smarty->assign('refunds', $refunds);
        $this->context->smarty->assign('payment_id', $paymentId);
        return $this->display(__FILE__, '/views/templates/admin/refunds_link.tpl');
    }

    /**
     * @throws PrestaShopDatabaseException
     * @throws PrestaShopException
     * @throws LocalizationException
     */
    public function hookDisplayAdminOrderTabContent($params)
    {
        if (!$this->showRefundsPanel($params)) {
            return null;
        }

        $orderId = $params['id_order'];
        $order = new Order($orderId);
        $paymentId = $this->getPaymentIdFromOrder($order);
        $showRefundCreationForm = $this->showRefundCreationForm($paymentId);

        $showSuccess = false;
        $showError = false;
        $alertMessage = '';
        $locale = $this->context->getCurrentLocale();

        $refundType = Tools::getValue('conpay_refund_type');
        $refundAmount = str_replace(',', '.', $refundType === 'FULL' ? $order->total_paid : Tools::getValue('conpay_refund_amount'));
        $refundCurrency = $locale->getPriceSpecification(Currency::getCurrency($order->id_currency)['iso_code'])->getCurrencySymbol();
        $reason = empty(Tools::getValue('conpay_refund_reason')) ? $this->l('Refund reason') : Tools::getValue('conpay_refund_reason');

        if ($_SERVER['REQUEST_METHOD'] === 'POST' && Tools::getValue('conpay_refund_submit') && isset($_SESSION['conpay_form_token']) && Tools::getValue('conpay_token_check') === $_SESSION['conpay_form_token']) {
            if (!is_numeric($refundAmount)) {
                $showError = true;
                $alertMessage = $this->l('Wrong refund amount');
            } else if ($refundAmount > $order->total_paid) {
                $showError = true;
                $alertMessage = $this->l('The refund amount you entered is greater than paid amount');
            } else {
                try {
                    $refundId = $this->conOrderRefund($refundAmount, $order, $reason)->getId();
                    RefundRepository::createRefund($orderId, $refundId, $paymentId, $refundType, $refundAmount, $order->id_currency);
                    if ($refundType === 'FULL') {
                        $showRefundCreationForm = false;
                    }

                    $history = new OrderHistory();
                    $history->id_order = (int)$orderId;
                    $history->id_employee = (int)$this->context->employee->id;
                    $history->changeIdOrderState($this->getConfigurationValue('PS_OS_REFUND'), $orderId);
                    $history->addWithemail(true, array());

                    $showSuccess = true;
                    $alertMessage = sprintf($this->l('Successfully created refund'), $refundId);
                } catch (CKPL\Pay\Exception\Http\HttpConflictException | CKPL\Pay\Exception\Http\HttpNotFoundException $e) {
                    $showError = true;
                    $alertMessage = $e->getTranslatedMessage($this->context->language->iso_code) ?? $this->getRefundCreationProblemMessage();
                } catch (Exception $e) {
                    $showError = true;
                    $alertMessage = $this->getRefundCreationProblemMessage();
                }
            }
        }

        $formToken = (string)mt_rand();
        $_SESSION['conpay_form_token'] = $formToken;

        $refunds = RefundRepository::getRefundsByOrderId($params['id_order']);

        $this->context->smarty->assign([
            'conpay_show_refund_creation_form' => $showRefundCreationForm,
            'conpay_refund_type' => $refundType,
            'conpay_refund_amount' => is_numeric($refundAmount) ? $locale->formatNumber($refundAmount) : $refundAmount,
            'conpay_refund_currency' => $refundCurrency,
            'conpay_refund_full_amount' => $locale->formatNumber($order->total_paid),
            'conpay_refund_reason' => $reason,
            'conpay_refunds' => $refunds,
            'conpay_form_token' => $formToken,
            'show_success' => $showSuccess,
            'show_error' => $showError,
            'alert_message' => $alertMessage
        ]);
        return $this->display(__FILE__, '/views/templates/admin/refunds_tab.tpl');
    }

    /**
     * @param Order $order
     * @return string|null
     */
    private function getPaymentIdFromOrder($order)
    {
        /** @var OrderPayment $orderPayment */
        foreach ($order->getOrderPayments() as $orderPayment) {
            if (!empty($orderPayment->transaction_id)) {
                return $orderPayment->transaction_id;
            }
        }
        return null;
    }

    /**
     * @param string|null $paymentId
     * @return bool
     * @throws PrestaShopDatabaseException
     */
    private function showRefundCreationForm($paymentId)
    {
        if (empty($paymentId)) {
            return false;
        }
        return PaymentRepository::isRefundCreationAllowed($paymentId) && !RefundRepository::isAnyFullRefund($paymentId);
    }

    private function showRefundsPanel($params)
    {
        if (isset($params['id_order'])) {
            try {
                $order = new Order($params['id_order']);
            } catch (PrestaShopDatabaseException | PrestaShopException $e) {
                return false;
            }
            if ($order->module === Definitions::MODULE_NAME) {
                return true;
            }
        }
        return false;
    }

    /**
     * @param float $amount
     * @param Order $order
     *
     * @param string $reason
     * @return CreatedRefundResponseModel|string
     *
     * @throws ClientException
     * @throws \CKPL\Pay\Exception\Exception
     * @throws Exception
     */
    private function conOrderRefund($amount, $order, $reason)
    {
        try {
            $orderPayment = $order->getOrderPayments();

            $currency = Currency::getCurrency($order->id_currency);
            $sdk = $this->initializeConotoxiaPay();
            $refundBuilder = $sdk->refunds()->createRefundBuilder();
            $refund = $refundBuilder->setPaymentId($orderPayment[0]->transaction_id)
                ->setExternalRefundId((string)$order->id)
                ->setReason($reason)
                ->setValue((string)$amount)
                ->setCurrency($currency['iso_code'])
                ->setIntegrationPlatform($this->getShopVersions())
                ->setAcceptLanguage($this->getAcceptLanguage())
                ->getRefund();
            return $sdk->refunds()
                ->makeRefund($refund);
        } catch (CKPL\Pay\Exception\ClientException | \CKPL\Pay\Exception\Exception $e) {
            PrestaShopLogger::addLog('ConotoxiaPay - refund creation problem: ' . $e->getMessage(), 3, $e->getCode(), 'Order', $order->id, true);
            throw $e;
        }
    }

    /**
     * @return string
     */
    public function getShopVersions()
    {
        return 'PRESTA=' . _PS_VERSION_ . ';PLUGIN=' . $this->version;
    }

    /**
     * @return string
     * @throws PrestaShopException
     */
    private function getRefundCreationProblemMessage()
    {
        return sprintf($this->l('Refund creation problem'), $this->generateLogsLink($this->l('Logs')));
    }

    /**
     * @return array
     */
    private function getConfigurationData()
    {
        return [
            ConfigurationInterface::CLIENT_ID => Configuration::get(Definitions::CLIENT_ID),
            ConfigurationInterface::CLIENT_SECRET => Configuration::get(Definitions::CLIENT_SECRET),
            ConfigurationInterface::POINT_OF_SALE => Configuration::get(Definitions::POINT_OF_SALE),
            ConfigurationInterface::PRIVATE_KEY => Configuration::get(Definitions::PRIVATE_KEY),
            ConfigurationInterface::PUBLIC_KEY => Configuration::get(Definitions::PUBLIC_KEY),
        ];
    }

    /**
     * @param $key
     * @return mixed
     * @throws Exception
     */
    private function getConfigurationValue($key)
    {
        /** @var PrestaShop\PrestaShop\Adapter\Configuration $configuration */
        $configuration = $this->get('prestashop.adapter.legacy.configuration');
        return $configuration->get($key);
    }

    /**
     * @param $key
     * @param $value
     * @throws Exception
     */
    private function setConfigurationValue($key, $value)
    {
        /** @var PrestaShop\PrestaShop\Adapter\Configuration $configuration */
        $configuration = $this->get('prestashop.adapter.legacy.configuration');
        $configuration->set($key, $value);
    }

    /**
     * @param $name
     * @return string
     */
    public static function buildTemplatePath($name)
    {
        return 'module:conotoxiapay/views/templates/front/' . $name . '.tpl';
    }

    /**
     * @param array $orderPayments
     * @param string $paymentId
     * @return bool
     */
    public static function isValidOrder($orderPayments, $paymentId)
    {
        if ($orderPayments === null || empty($orderPayments)) {
            return false;
        }
        foreach ($orderPayments as $payment) {
            if ($payment->transaction_id === $paymentId) {
                return true;
            }
        }
        return false;
    }

    /**
     * @return string|null
     */
    private function getAcceptLanguage(): ?string
    {
        return $_SERVER['HTTP_ACCEPT_LANGUAGE'];
    }
}
