<?php

use CKPL\Pay\Model\Response\CreatedPaymentResponseModel;
use PrestaShop\Module\ConotoxiaPay\Configure\Configurator;
use PrestaShop\Module\ConotoxiaPay\Repository\PaymentRepository;

class ConotoxiaPayPaymentModuleFrontController extends ModuleFrontController
{
    public $ssl = true;

    /**
     * @var bool
     */
    private $error;

    /**
     * @throws PrestaShopException
     * @throws Exception
     *
     * @see FrontController::initContent()
     */
    public function initContent()
    {
        parent::initContent();
        $this->error = false;

        if (Tools::getValue('payByConPay')) {
            try {
                $payment = $this->pay();
                Tools::redirect($payment->getApproveUrl());
                return;
            } catch (Exception $e) {
                $this->error = true;
                PrestaShopLogger::addLog('ConotoxiaPay - payment creation problem: ' . $e->getMessage() . ' trace: ' . $e->getTraceAsString(), 3, $e->getCode(), 'Order', $this->module->currentOrder, true);
            }
        }
        $this->prepareTemplate();
    }

    /**
     * @return CreatedPaymentResponseModel
     * @throws Exception
     *
     * @throws PrestaShopException
     */
    private function pay()
    {
        $waitingStatus = Configurator::getWaitingStatus();
        $this->module->validateOrder(
            $this->context->cart->id,
            $waitingStatus,
            $this->context->cart->getOrderTotal(true, Cart::BOTH),
            $this->module->displayName,
            null,
            array(),
            (int)$this->context->cart->id_currency,
            false,
            $this->context->cart->secure_key,
            Context::getContext()->shop->id
                ? new Shop((int)Context::getContext()->shop->id) : null
        );

        $order = new Order($this->module->currentOrder);

        /** @var CKPL\Pay\Pay $sdk */
        $sdk = $this->module->initializeConotoxiaPay();
        $paymentBuilder = $sdk->payments()->createPaymentBuilder();
        $paymentAmount = $this->buildPaymentAmount($paymentBuilder);

        $createdPayment = $sdk->payments()->makePayment(
            $paymentBuilder
                ->setExternalPaymentId($order->id)
                ->setAmount($paymentAmount)
                ->setDescription($order->id)
                ->setIntegrationPlatform($this->module->getShopVersions())
                ->setAcceptLanguage($this->getAcceptLanguage())
                ->denyPayLater()
                ->getPayment()
        );

        PaymentRepository::createPayment($order->id, $createdPayment->getPaymentId());
        $order->addOrderPayment($paymentAmount->getValue(), $this->l('Conotoxia Pay'), $createdPayment->getPaymentId());

        return $createdPayment;
    }

    /**
     * @param CKPL\Pay\Definition\Payment\Builder\PaymentBuilderInterface $paymentBuilder
     *
     * @return CKPL\Pay\Definition\Amount\AmountInterface
     * @throws Exception
     */
    private function buildPaymentAmount($paymentBuilder)
    {
        $totalAmount = $this->context->cart->getOrderTotal(true, Cart::BOTH);
        $currency = new Currency((int)$this->context->cart->id_currency);

        return $paymentBuilder->createAmountBuilder()
            ->setValue(number_format($totalAmount, $currency->precision, '.', ''))
            ->setCurrency($currency->iso_code)
            ->getAmount();
    }

    /**
     * @throws PrestaShopException
     * @throws Exception
     */
    private function prepareTemplate()
    {
        $presentedCart = $this->cart_presenter->present($this->context->cart);
        $this->context->smarty->assign([
            'subtotalShipping' => $presentedCart['subtotals']['shipping'],
            'subtotalProducts' => $presentedCart['subtotals']['products'],
            'total' => $presentedCart['totals']['total'],
            'productsCount' => $this->context->cart->nbProducts(),
            'smallLogo' => $this->module->getLogo(),
            'showError' => $this->error
        ]);
        $this->setTemplate(ConotoxiaPay::buildTemplatePath('payment_execution'));
    }

    public function postProcess()
    {
        if (!Module::isEnabled($this->module->name)) {
            PrestaShopLogger::addLog('ConotoxiaPay - payment creation problem: module is disabled', 2, null, 'Order', $this->module->currentOrder, true);
            die($this->module->l('Payment method is not available.', 'conotoxiapay'));
        }
        if (!$this->module->active) {
            PrestaShopLogger::addLog('ConotoxiaPay - payment creation problem: module is not active', 2, null, 'Order', $this->module->currentOrder, true);
            die($this->module->l('Payment method is not active.', 'conotoxiapay'));
        }
        if (!$this->module->isConfigurationCompleted()) {
            PrestaShopLogger::addLog('ConotoxiaPay - payment creation problem: configuration is not completed', 3, null, 'Order', $this->module->currentOrder, true);
            die($this->module->l('Payment method is not configured.', 'conotoxiapay'));
        }

        if (!$this->context->cart->id) {
            Tools::redirect($this->context->link->getPageLink('cart'));
        }
        if ($this->context->cart->id_customer == 0) {
            Tools::redirect($this->context->link->getPageLink('order'));
        }
    }

    /**
     * @return string|null
     */
    private function getAcceptLanguage(): ?string
    {
        return $_SERVER['HTTP_ACCEPT_LANGUAGE'];
    }
}
